/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2004
 *
 * Author(s):
 *	Volker Fischer, Stephane Fillod
 *
 * Adapted for ham sstv use (TX only) Ties Bos - PA0MBO
 *
 * Description:
 *
 * 11/10/2004 Stephane Fillod
 *	- QT translation
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#ifdef _WIN32
# ifdef _WIN32_WINNT
#  undef _WIN32_WINNT
# endif
# define _WIN32_WINNT 0x0400
# include <windows.h>
#endif

#include "../GlobalDefinitions.h"
#include "../DrmReceiver.h"
#include "../DrmTransmitter.h"
#include "../DrmSimulation.h"
#include "../util/Settings.h"

#ifdef USE_QT_GUI
# include <qapplication.h>
# include <qthread.h>
# include <qmessagebox.h>
# include "fdrmdialog.h"
# include "TransmDlg.h"
#endif
#include <iostream>

/* Implementation *************************************************************/
#ifdef USE_QT_GUI
/******************************************************************************\
* Using GUI with QT                                                            *
\******************************************************************************/

int
main(int argc, char **argv)
{
	/* create app before running Settings.Load to consume platform/QT parameters */
        printf("Voor QAPP create \n");
	QApplication app(argc, argv);
        /* pa0mbo  */
	printf("Na QApplication app call\n");
	
#if defined(__APPLE__) &&  QT_VERSION >= 0x030000
	/* find plugins on MacOs when deployed in a bundle */
	app.setLibraryPaths(app.applicationDirPath()+"../PlugIns");
#endif

	/* Load and install multi-language support (if available) */
	QTranslator translator(0);
	if (translator.load("dreamtr"))
		app.installTranslator(&translator);

	CDRMSimulation DRMSimulation;

	/* Call simulation script. If simulation is activated, application is
	   automatically exit in that routine. If in the script no simulation is
	   activated, this function will immediately return */
	DRMSimulation.SimScript();
         printf("na simscript call\n");

	CSettings Settings;
	/* Parse arguments and load settings from init-file */
	Settings.Load(argc, argv);
        printf("na Settings.load in main \n");

	try
	{

#ifdef _WIN32
		/* works for both transmit and receive. GUI is low, working is normal.
		 * the working thread does not need to know what the setting is.
		 */
			if (Settings.Get("GUI", "processpriority", TRUE))
			{
				/* Set priority class for this application */
				SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);

				/* Low priority for GUI thread */
				SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_LOWEST);
				Settings.Put("GUI", "processpriority", TRUE);
			}
			else
				Settings.Put("GUI", "processpriority", FALSE);
#endif

		string mode = Settings.Get("command", "mode", string("receive"));
		if (mode == "receive")
		{
			CDRMReceiver DRMReceiver;
                        printf("in mode receive \n");
			/* First, initialize the working thread. This should be done in an extra
			   routine since we cannot 100% assume that the working thread is
			   ready before the GUI thread */

			DRMReceiver.LoadSettings(Settings);

			if (Settings.Get("AM Dialog", "visible", false))
				DRMReceiver.SetReceiverMode(RM_AM);
			else
				DRMReceiver.SetReceiverMode(RM_DRM);

			DRMReceiver.Init();

			FDRMDialog MainDlg(DRMReceiver, Settings, 0, 0, FALSE, Qt::WStyle_MinMax);

			/* Start working thread */
			DRMReceiver.start();

			/* Set main window */
			app.setMainWidget(&MainDlg);

			app.exec();

			DRMReceiver.SaveSettings(Settings);
		}
		else if(mode == "transmit")
		{
                        printf("In transmitmode \n");
			TransmDialog MainDlg(Settings, 0, 0, FALSE, Qt::WStyle_MinMax);

			/* Set main window */
			app.setMainWidget(&MainDlg);

			/* Show dialog */
			MainDlg.show();
			printf("na MainDlg show \n");
			app.exec();
		}
		else
		{
			QMessageBox::information(0, "Dream", Settings.UsageArguments(argv).c_str());
			exit(0);
		}
	}

	catch(CGenErr GenErr)
	{
		ErrorMessage(GenErr.strError);
	}
	catch(string strError)
	{
		ErrorMessage(strError);
	}
	catch(char *Error)
	{
		ErrorMessage(Error);
	}

	/* Save settings to init-file */
	Settings.Save();

	return 0;
}

/* Implementation of global functions *****************************************/

void
ErrorMessage(string strErrorString)
{
	/* Workaround for the QT problem */
	string strError = "The following error occured:\n";
	strError += strErrorString.c_str();
	strError += "\n\nThe application will exit now.";

#ifdef _WIN32
	MessageBoxA(NULL, strError.c_str(), "Dream",
			   MB_SYSTEMMODAL | MB_OK | MB_ICONEXCLAMATION);
#else
	perror(strError.c_str());
#endif

/*
// Does not work correctly. If it is called by a different thread, the
// application hangs! FIXME
	QMessageBox::critical(0, "Dream",
		QString("The following error occured:<br><b>") +
		QString(strErrorString.c_str()) +
		"</b><br><br>The application will exit now.");
*/
	exit(1);
}
#else /* USE_QT_GUI */
/******************************************************************************\
* No GUI                                                                       *
\******************************************************************************/

int
main(int argc, char **argv)
{
        string  picname ;
        string picformat ;
        int i , nr_devin, nr_devout, iId;
        string MyCall;
        ERobMode RobMod;
        ESpecOcc BW ;
        CParameter::ESymIntMod eNewDepth;
        ECodScheme eNewScheme;
        CMSCProtLev eNewMSCPre;
        size_t pos;
        CParameter* TransmParam ;
        CService Service;
	try
	{
		CSettings Settings;
		Settings.Load(argc, argv);
           //     printf("Na settings.load \n");
		{
			CDRMTransmitter DRMTransmitter;
                        nr_devin = Settings.Get("sounddevices", "devin", nr_devin);
                        nr_devout = Settings.Get("sounddevices", "devout", nr_devout);
                        DRMTransmitter.GetSoundInInterface()->SetDev(nr_devin);
                        DRMTransmitter.GetSoundOutInterface()->SetDev(nr_devout);
                        DRMTransmitter.GetAudSrcEnc()->ClearPicFileNames();
                        TransmParam = DRMTransmitter.GetParameters();
                        if (argc > 1)
                        {
                            for (i=1; i < argc ; i++)
                            {
                             printf("arg %d is %s\n", i, argv[i]);
                              picname = (string) argv[i] ;
                              pos =  picname.rfind(".");
                              picformat= picname.substr(pos+1);
                              DRMTransmitter.GetAudSrcEnc()->SetPicFileName(picname, picformat);
                            }
                        }
            //            printf("In main na reading cmdline args \n");
                        iId = Settings.Get("mode", "robustness", iId);
                        switch (iId)
                        {
                           case 0 :
                              RobMod = RM_ROBUSTNESS_MODE_A;
                              break;
                           case 1 :
                              RobMod = RM_ROBUSTNESS_MODE_B ;
                              break ;
                           case 2 :
                              RobMod = RM_ROBUSTNESS_MODE_E ;
                              break;
                         }
                        iId = Settings.Get("mode", "BW", iId);
                        switch (iId)
                        {
                           case 0 :
                              BW = SO_0;
                              break;
                           case 1 :
                              BW = SO_1 ;
                              break ;
                         }
                        TransmParam->InitCellMapTable(RobMod , BW);  
             //            printf("In main na InitCellMapTable \n");     
                        iId = Settings.Get("mode", "intlvldepth", iId);
                        switch (iId)
                        {
                            case 0 :
                               eNewDepth = CParameter::SI_LONG;
                               break ;
                            case 1 :
                               eNewDepth = CParameter::SI_SHORT;
                               break;
                        }
                        TransmParam->SetInterleaverDepth(eNewDepth);       
                        iId = Settings.Get("mode", "qam", iId);
                        switch (iId)
                        {
                           case 64 :
                              eNewScheme = CS_3_SM;
                              break;
                           case 16 :
                              eNewScheme = CS_2_SM ;
                              break ;
                           case 4 :
                              eNewScheme = CS_1_SM ;
                              break;
                         }
                         TransmParam->SetMSCCodingScheme(eNewScheme);
                        iId = Settings.Get("mode", "protlev", iId);
                        switch (iId)
                        {
                           case 0 :
                              eNewMSCPre.iPartB =  0;
                              break;
                           case 1 :
                              eNewMSCPre.iPartB = 1 ;
                              break ;
                         }
                        TransmParam->SetMSCProtLev(eNewMSCPre, FALSE); 
                        Service.iServiceDescr=0;
                        Service.iServiceID=0;
                        Service.iLanguage=5;
                        MyCall = Settings.Get("callsign", "mycall", MyCall);
                        Service.strLabel=MyCall;
                        TransmParam->SetServiceParameters(0,Service);
              //          printf("net voor DRMTransmitter start\n"); 
			DRMTransmitter.Start();
		}
	}
	catch(CGenErr GenErr)
	{
		ErrorMessage(GenErr.strError);
	}
    //    printf("Net voor return uit main \n");

	return 0;
}

void
ErrorMessage(string strErrorString)
{
	perror(strErrorString.c_str());
}
#endif /* USE_QT_GUI */

void
DebugError(const char *pchErDescr, const char *pchPar1Descr,
		   const double dPar1, const char *pchPar2Descr, const double dPar2)
{
	FILE *pFile = fopen("test/DebugError.dat", "a");
	fprintf(pFile, pchErDescr);
	fprintf(pFile, " ### ");
	fprintf(pFile, pchPar1Descr);
	fprintf(pFile, ": ");
	fprintf(pFile, "%e ### ", dPar1);
	fprintf(pFile, pchPar2Descr);
	fprintf(pFile, ": ");
	fprintf(pFile, "%e\n", dPar2);
	fclose(pFile);
	printf("\nDebug error! For more information see test/DebugError.dat\n");
	exit(1);
}
